<?php
namespace FiloBlu\ExtViSualMerchandiser\Model\Category;

use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\DB\Select;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Zend_Json_Exception;

/**
 * Class Products
 * @package FiloBlu\ExtViSualMerchandiser\Model\Category
 */
class Products extends \Magento\VisualMerchandiser\Model\Category\Products
{
    /** @var ScopeConfigInterface */
    protected $scopeConfig;

    /** @var StoreManagerInterface  */
    private $storeManager;

    /**
     * Compatibility method
     *
     * @return StoreManagerInterface
     */
    protected function getStoreManager()
    {
        if ($this->storeManager === null) {
            $this->storeManager = ObjectManager::getInstance()->get(StoreManagerInterface::class);
        }
        return $this->storeManager;
    }

    /**
     * Compatibility method
     *
     * @return ScopeConfigInterface
     */
    protected function getScopeConfig()
    {
        if ($this->scopeConfig === null) {
            $this->scopeConfig = ObjectManager::getInstance()->get(ScopeConfigInterface::class);
        }

        return $this->scopeConfig;
    }

    /**
     * @param int $categoryId
     * @param int|null $store
     * @param int|null $productPositions
     * @return Collection
     * @throws LocalizedException
     * @throws Zend_Json_Exception
     */
    public function getCollectionForGrid($categoryId, $store = null, $productPositions = null)
    {
        $storeId = null;

        if ($store === null) {
            $storeId = $this->getStoreManager()->getStore()->getId();
        } else {
            $storeId = $store;
        }

        /** @var Collection $collection */
        $collection = $this->getFactory()->create()
            ->getCollection()
            ->addAttributeToSelect([
                'sku',
                'name',
                'status',
                'price',
                'small_image',
                'image',
                'thumbnail'
            ]);

        if ($store !== null) {
            $collection->addStoreFilter($storeId);
        }

        $collection->getSelect()
            ->where('at_position.category_id = ?', $categoryId);

        if ($this->_moduleManager->isEnabled('Magento_CatalogInventory')) {
            if ($this->getStockId() == Stock::DEFAULT_STOCK_ID) {
                $collection->joinField(
                    'stock',
                    'cataloginventory_stock_item',
                    'qty',
                    'product_id=entity_id',
                    ['stock_id' => $this->getStockId()],
                    'left'
                );
            } else {
                $collection->joinField(
                    'stock',
                    'inventory_stock_' . $this->getStockId(),
                    'quantity',
                    'sku=sku',
                    null,
                    'left'
                );
            }
        }

        $cache = $this->_cache->getPositions($this->_cacheKey);

        if ($cache === false) {
            $collection->joinField(
                'position',
                'catalog_category_product',
                'position',
                'product_id=entity_id',
                null,
                'left'
            );
            $collection->setOrder('position', $collection::SORT_ORDER_ASC);

            // Cache the positions initially
            $_collection = clone $collection;

            $positions = [];
            $idx = 0;
            foreach ($_collection as $item) {
                $positions[$item->getId()] = $idx;
                $idx++;
            }

            $this->_cache->saveData($this->_cacheKey, $positions);
        } else {
            $collection->getSelect()
                ->reset(Select::WHERE)
                ->reset(Select::HAVING);

            $collection->addAttributeToFilter('entity_id', ['in' => array_keys($cache)]);
        }

        return $collection;
    }

    /**
     * @return int
     */
    protected function getStockId()
    {
        $stock = $this->getScopeConfig()->getValue('visualmerchandiser/options/default_stock', ScopeInterface::SCOPE_STORE);
        if ($stock) {
            return $stock;
        }
        return Stock::DEFAULT_STOCK_ID;
    }
}
