<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\ExtViSualMerchandiser\Model\Sorting;

use FiloBlu\ExtViSualMerchandiser\Helper\ConfigHelper;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\DB\Select;
use Magento\Store\Model\ScopeInterface;
use Zend_Db_Expr;

class LowStockTop extends \Magento\VisualMerchandiser\Model\Sorting\LowStockTop
{
    const XML_PATH_MIN_STOCK_THRESHOLD = 'visualmerchandiser/options/minimum_stock_threshold';

    /**
     * @param Collection $collection
     * @return Collection
     */
    public function sort(
        Collection $collection
    ) {
        if (!$this->moduleManager->isEnabled('Magento_CatalogInventory')) {
            return $collection;
        }

        $minStockThreshold = (int)$this->scopeConfig->getValue(self::XML_PATH_MIN_STOCK_THRESHOLD);

        $baseSet = clone $collection;
        $finalSet = clone $collection;

        $collection->joinField(
            'child_id',
            'catalog_product_relation',
            'child_id',
            'parent_id=entity_id',
            null,
            'left'
        );

        if ($this->getStockId() == Stock::DEFAULT_STOCK_ID) {
            $collection->joinField(
                'qty',
                'cataloginventory_stock_item',
                'qty',
                'product_id=entity_id',
                ['stock_id' => $this->getStockId()],
                'left'
            );

            $collection->joinField(
                'parent_qty',
                'cataloginventory_stock_item',
                'qty',
                'product_id=child_id',
                ['stock_id' => $this->getStockId()],
                'left'
            );

            $collection->getSelect()
                ->columns('IF(SUM(`at_parent_qty`.`qty`), SUM(`at_parent_qty`.`qty`), SUM(`at_qty`.`qty`)) as final_qty')
                ->group('entity_id')
                ->having('final_qty <= ?', $minStockThreshold)
                ->reset(Select::ORDER)
                ->order('final_qty ' . $collection::SORT_ORDER_ASC);
        } else {
            $collection->joinField(
                'qty',
                'inventory_stock_' . $this->getStockId(),
                'quantity',
                'sku=sku',
                null,
                'left'
            );

            $collection->getSelect()
                ->columns('SUM(`at_qty`.`quantity`) as final_qty')
                ->group('entity_id')
                ->having('final_qty <= ?', $minStockThreshold)
                ->reset(Select::ORDER)
                ->order('final_qty ' . $collection::SORT_ORDER_ASC);
        }

        /*$collection->getSelect()
            ->columns('IF(SUM(`at_parent_qty`.`qty`), SUM(`at_parent_qty`.`qty`), SUM(`at_qty`.`qty`)) as final_qty')
            ->group('entity_id')
            ->having('final_qty <= ?', $minStockThreshold)
            ->reset(Select::ORDER)
            ->order('final_qty '.$collection::SORT_ORDER_ASC);*/

        $resultIds = [];

        $collection->load();

        foreach ($collection as $item) {
            $resultIds[] = $item->getId();
        }

        $ids = array_unique(array_merge($resultIds, $baseSet->getAllIds()));

        $finalSet->getSelect()
            ->reset(Select::ORDER)
            ->reset(Select::WHERE);

        $finalSet->addAttributeToFilter('entity_id', ['in' => $ids]);
        if (count($ids)) {
            $finalSet->getSelect()->order(new Zend_Db_Expr('FIELD(e.entity_id, ' . implode(',', $ids) . ')'));
        }

        return $finalSet;
    }

    /**
     * @return int
     */
    protected function getStockId()
    {
        $stock = $this->scopeConfig->getValue(ConfigHelper::CONFIG_DEFAULT_STOCK, ScopeInterface::SCOPE_STORE);
        if ($stock) {
            return $stock;
        }
        return Stock::DEFAULT_STOCK_ID;
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return __('Move low stock to top');
    }
}
